<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use \Setforms2\Shared\ORM\DateTime;

class Pla00100ElaboracaoDoCardapioController extends \Teknisa\GER\Controllers\TeknisaCrud {

    const TIPOS_DE_PRATO         = 'cnfcames_tipoprat';
    const CONFIGURACAO_SERVICO   = 'confcard_tipoprat';
    const COLUNA_DATA_REFERENCIA = 'DTREFECNFMES';
    const FERIFILI               = 'ferifili';
    const FERICARD               = 'fericard';
    const CARDADIA_TURNOSERV     = 'cardadia_turnoserv';
    const CARDADIA_PONTO_ENTREGA = 'cardadia_ponto_entrega';
    const PONTO_ENTREGA_DESTINO  = 'ponto_entrega_destino';
    const SERVICO                = 'servico';
    const PRATOS_CARDAPIO        = 'pratos_cardapio';
    const INDEX_FERIADO          = 7;

    protected $paramgeral;
    protected $elaboracaoDeCardapio;
    protected $parameterBagColumns;
    protected $environment;
    protected $estadoDoCardapio;
    protected $tipoDeVisualizacao;
    protected $dataSourceName;
    protected $statusCardapio;
    protected $compcarddel;
    protected $entityManager;
    protected $authController;
    protected $util;
    protected $libUtilities;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $elaboracaoDeCardapio, $util, $newCode, $entityManager, $authController,$libUtilities) {
        $this->crudService          = $crudService;
        $this->parameterBag         = $parameterBag;
        $this->dataSourceManager    = $dataSourceManager;
        $this->environment          = $environment;
        $this->elaboracaoDeCardapio = $elaboracaoDeCardapio;
        $this->entityManager        = $entityManager;
        $this->authController       = $authController;
        $this->util                 = $util;
        $this->libUtilities         = $libUtilities;
    }

    public function retornaDadosDoCardapio(DTO\Request $request, DTO\Response $response) {
        try {
            

            $conditions = $request->getFilterCriteria()->getConditions();

            $cdfilial = $conditions[0]['value'];
            //coleta de dados
             $this->libUtilities->saveCollectedData($cdfilial,'CARDAPIO');
             
            $cdservfili = $conditions[1]['value'];
            $dtrefecnfmes = $conditions[2]['value'];
            $tipoCusto = $conditions[3]['value'];
            $dataReferencia = empty($conditions[4]['value']) ? null : $conditions[4]['value'];
            
            $paramplaneja = $this->elaboracaoDeCardapio->retornaParamplaneja($cdfilial);
            if(!$this->checkCustoPadrao($paramplaneja, $cdfilial, $cdservfili, $dtrefecnfmes, $response))
                return false;

            $this->paramgeral = $this->elaboracaoDeCardapio->retornaParamgeral();
            $this->paramgeral['QTDIABLOQCAR'] = ($this->paramgeral['QTDIABLOQCAR'] ? $this->paramgeral['QTDIABLOQCAR'] : 0);
            $dataBloqueio = $this->elaboracaoDeCardapio->retornaDataProcBloqueio($cdfilial, $this->paramgeral['QTDIABLOQCAR']);
            $tipoOperador = $this->elaboracaoDeCardapio->retornaTipoOperador();
            $cardapioDescentralizado = $this->elaboracaoDeCardapio->retornaTipoDeCardapio($cdfilial, $cdservfili);

            $tiposDePrato = $this->elaboracaoDeCardapio->retornaTiposDePrato(new FilterCriteria(self::TIPOS_DE_PRATO), $cdfilial, $cdservfili, $dtrefecnfmes);
            
            $dataSet = $this->retornaCardapioValidado($conditions, $cardapioDescentralizado, $tipoOperador, $dataBloqueio, $dtrefecnfmes, $tiposDePrato);
            $this->elaboracaoDeCardapio->insertCardames($cdfilial, $cdservfili, $dtrefecnfmes);

            $this->criaStatusCardapio($this->tipoDeVisualizacao, $this->estadoDoCardapio, $tipoOperador, $cardapioDescentralizado);
            $this->statusCardapio['data_proc_bloqueio'] = $dataBloqueio->format('d/m/Y');
            $this->statusCardapio['OPERREVISAOREC'] = $this->elaboracaoDeCardapio->retornaOperadorGrupoRevisao();

            $pratosDoCardapio = $this->retornaPratosDoCardapio($cdfilial, $cdservfili, $dtrefecnfmes, $tipoCusto, $dataReferencia);

            $comensaisDiasCardapio = $this->elaboracaoDeCardapio->retornaComensaisDiasCardapio($cdfilial, $cdservfili, $dtrefecnfmes);

            $feriadosFilial = $this->elaboracaoDeCardapio->retornaDiasFeriadoFilial(new FilterCriteria(self::FERIFILI), $cdfilial, $dtrefecnfmes);
            $feriadosCardapio = $this->elaboracaoDeCardapio->retornaDiasFeriadoCardapio(new FilterCriteria(self::FERICARD), $cdfilial, $dtrefecnfmes);

            $todosComensaisEstimados = $this->retornaTodosComensaisEstimados($cdfilial, $cdservfili);

            $response->addDataSet($dataSet);
            $response->addDataSet($pratosDoCardapio);
            $response->addDataSet($comensaisDiasCardapio);

            if ($this->compcarddel) {
                $response->addDataSet($this->compcarddel);
            }
            $response->addDataSet(new DataSet('status_cardapio', $this->statusCardapio));
            $response->addDataSet(new DataSet('paramplaneja', $this->retornaParametrosPlanejamento($paramplaneja)));
            $response->addDataSet(new DataSet('feriados', array('filial' => $feriadosFilial, 'cardapio' => $feriadosCardapio)));
            $response->addDataSet(new DataSet('todosComensaisEstimados', $todosComensaisEstimados));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    /**
     * Método que verifica se o cárdapio é descentralizado ou livre, se possui elaboração,
     * se o serviço possui configuração, se a data do cardápio é valida comparada com a data
     * de processamento mais os dias de bloqueio, se o cardápio está liberado para filial.
     *
     * @param Array $conditions
     * @param String $cardapioDescentralizado (S, N)
     * @param String $tipoOperador (FILIAL ou MATRIZ)
     * @param DateTime $dataBloqueio
     * @param String $dataCardapio (d/m/Y)
     * @param DataSet $tiposDePrato
     *
     * @return Exception|DataSet
     */
    private function retornaCardapioValidado($conditions, $cardapioDescentralizado, $tipoOperador, $dataBloqueio, $dataCardapio, $tiposDePrato) {
        $dataCardapio = $this->elaboracaoDeCardapio->dateToDateTime($dataCardapio);

        $dataBloqueio = $this->elaboracaoDeCardapio->firstDayOfMonth($dataBloqueio->format('d/m/Y'));

        if ($cardapioDescentralizado == 'S') {

            $this->estadoDoCardapio = $this->elaboracaoDeCardapio->retornaEstadoDoCardapio($conditions[0]['value'], $conditions[1]['value'], $conditions[2]['value']);
            $this->tipoDeVisualizacao = $this->elaboracaoDeCardapio->retornaTipoDeVisualizacaoDescentralizada($conditions[0]['value'], $conditions[2]['value'], $this->estadoDoCardapio, $tipoOperador, $cardapioDescentralizado);
            $this->compcarddel = $this->elaboracaoDeCardapio->retornaCompcarddel($conditions[0]['value'], $conditions[1]['value']);

            if ($tiposDePrato->getRows() && $this->estadoDoCardapio == 'NAO_LIBERADO_FILIAL' && $tipoOperador == 'FILIAL') {

                throw new \Exception("Cardápio não foi liberado pela matriz");

            } else if (count($tiposDePrato->getRows()) == 0 && $dataBloqueio <= $dataCardapio) {
                
                return $this->retornaColunasPorServico($conditions);

            } else if (count($tiposDePrato->getRows()) == 0) {

                throw new \Exception("Não há configuração de cardápio para o mês");

            } else {

                return $tiposDePrato;
            }

        } else {

            $this->tipoDeVisualizacao = $this->elaboracaoDeCardapio->retornaTipoDeVisualizacaoLivre($conditions[0]['value'], $tipoOperador);

            if (count($tiposDePrato->getRows()) == 0 && $dataBloqueio <= $dataCardapio) {
                
                return $this->retornaColunasPorServico($conditions);

            } else if (count($tiposDePrato->getRows()) == 0) {

                throw new \Exception("Não há configuração de cardápio para o mês");

            } else {

                return $tiposDePrato;
            }

        }

    }

    public function criaStatusCardapio($tipoDeVisualizacao, $estadoDoCardapio = null, $tipoOperador, $cardapioDescentralizado){

        $this->statusCardapio = array(
            "TIPODEVISUALIZACAO" => $tipoDeVisualizacao,
            "TIPOOPERADOR" => $tipoOperador,
            "ESTADODOCARDAPIO" => $estadoDoCardapio,
            "TIPODECARDAPIO" => $cardapioDescentralizado,
        );
    }

    /**
     * Método para retornar apenas os parâmetros de planejamento que irão ser utilizados
     * no sistema. Converte as cores do Delphi de decimal para hexadecimal.
     *
     * @param Array $paramplaneja
     *
     * @return Array
     */
    private function retornaParametrosPlanejamento($paramplaneja) {

        return array(
            'IDCARDIAUTIL' => $paramplaneja['IDCARDIAUTIL'],
            'NRCORNALTE' => $this->TColorToHex($paramplaneja['NRCORNALTE']),
            'NRCORNUTIL' => $this->TColorToHex($paramplaneja['NRCORNUTIL']),
            'NRCORCARDREV' => $this->TColorToHex($paramplaneja['NRCORCARDREV']),
            'NRCORCARDESP' => $this->TColorToHex($paramplaneja['NRCORCARDESP']),
            'IDUTRPADCA' => $this->paramgeral['IDUTRPADCA'],
            'IDMANCORCARDALT' => $this->paramgeral['IDMANCORCARDALT'],
            'IDBLOALTRELO' => $paramplaneja['IDBLOALTRELO'],
            'QTDIABLOQCAR' => $this->paramgeral['QTDIABLOQCAR'],
            'IDPERCOMENSPRAT' => $this->paramgeral['IDPERCOMENSPRAT'],
            'CDGRUPPRODREVRECEITA' => $this->paramgeral['CDGRUPPRODREVRECEITA']
        );
    }

    public function TColorToHex($TColor) {

        $rgb = $this->TColorToRGB($TColor);

        return $this->RGBToHex($rgb['r'], $rgb['g'], $rgb['b']);

    }

    public function TColorToRGB($TColor) {
        $r = $TColor & 0xFF;
        $g = ($TColor >> 8) & 0xFF;
        $b = ($TColor >> 16) & 0xFF;

        return array('r' => $r, 'g' => $g, 'b' => $b);
    }

    public function RGBToHex($r, $g, $b) {

        $hex = "#";
        $hex.= str_pad(dechex($r), 2, "0", STR_PAD_LEFT);
        $hex.= str_pad(dechex($g), 2, "0", STR_PAD_LEFT);
        $hex.= str_pad(dechex($b), 2, "0", STR_PAD_LEFT);

        return $hex;
    }

    /**
     * Método para retornar as colunas do cardápio de acordo com a configuração do serviço.
     * Caso não exista, retorna exceção.
     *
     * @param DTO\Request $request
     *
     * @return Exception|DataSet
     */
    private function retornaColunasPorServico($conditions) {
        
        $filterCriteria = new FilterCriteria(self::CONFIGURACAO_SERVICO);
        $filterCriteria->addCondition('CDFILIAL', $conditions[0]['value']);
        $filterCriteria->addCondition('CDSERVFILI', $conditions[1]['value']);

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        
        $dataSet->setDataSourceName(self::TIPOS_DE_PRATO);
        
        if ($dataSet->getRows()) {
        
            foreach ($dataSet->getRows() as $row) {
                
                $row['DSCOMPLTPMES'] = $row['DSCOMPLTP'];
                $row['IDCONCNFMES'] = $row['IDCONCNFCARD'];
                $this->elaboracaoDeCardapio->insertCnfcames($row, $conditions[2]['value']);
            }
            
            return $dataSet;

        } else {

            throw new \Exception("Não há configuração de cardápio para o mês nem para o serviço");

        }

    }

    private function retornaPratosDoCardapio($cdfilial, $cdservfili, $dataInicial, $tipoCusto, $dataReferencia) {

        $dataFinal = $this->elaboracaoDeCardapio->retornaUltimaDataMesString($dataInicial, 'd/m/Y');
        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DTREFECNFMES', 'NRORG', 'DATAINICIAL', 'DATAFINAL', 'DATACUSTO', 'TIPOCUSTO');

        $filterCriteria = new FilterCriteria(self::PRATOS_CARDAPIO);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTREFECNFMES', $dataInicial);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('DATAINICIAL', $dataInicial);
        $filterCriteria->addCondition('DATAFINAL', $dataFinal);
        $filterCriteria->addCondition('DATACUSTO', $dataReferencia);
        $filterCriteria->addCondition('TIPOCUSTO', $tipoCusto);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        return $this->dataSourceManager->findBy($newFilterCriteria);

    }

    /**
     * Método que remove uma condição de um array de condições de uma requisição
     *
     * @param String $columnName
     * @param Array $conditions
     *
     * @return Array
     */
    private function removeCondition($columnName, array $conditions) {

        $conditionsColumns = array();

        foreach ($conditions as $condition) {
            $conditionsColumns[] = $condition['columnName'];
        }

        $key = array_search($columnName, $conditionsColumns);
        unset($conditions[$key]);

        return $conditions;

    }

    public function retornaTurnos(DTO\Request $request, DTO\Response $response) {

        $conditions = $request->getFilterCriteria()->getConditions();

        $cdfilial = $conditions[0]['value'];
        $cdservfili = $conditions[1]['value'];
        $dtcardapio = $conditions[2]['value'];
        $diaDaSemana = $conditions[3]['value'];

        $turnosCardadia = $this->retornaTurnosCardadia($cdfilial, $cdservfili, $dtcardapio);

        if (count($turnosCardadia->getRows())==0) {

            $turnosCardadia = $this->retornaTurnosServico($cdfilial, $cdservfili, $diaDaSemana);
            $turnosCardadia = new DataSet('turnos', $turnosCardadia);

        }

        $turnosCardadia->setDataSourceName('turnos');

        $response->addDataSet($turnosCardadia);
    }

    public function retornaTurnosCardadia($cdfilial, $cdservfili, $dtcardapio) {

        $filterCriteria = new FilterCriteria(self::CARDADIA_TURNOSERV);
        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DTCARDAPIO');

        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTCARDAPIO', $dtcardapio);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

        return $dataSet;
    }

    public function retornaTurnosServico($cdfilial, $cdservfili, $diaDaSemana) {

        $filterCriteria = new FilterCriteria('turnoserv');

        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        return $this->retornaTurnoDiaDaSemana($diaDaSemana, $dataSet->getRows());

    }

    public function retornaTurnoDiaDaSemana($diaDaSemana, $turnos) {

        $turnosFiltrados = array();
        $diaDaSemana = strlen($diaDaSemana) > 1 ? $diaDaSemana : $this->elaboracaoDeCardapio->retornaColunaSemana($diaDaSemana);
        $diaDaSemanaTurno = 'QTCOMESTT'.$diaDaSemana;

        foreach ($turnos as $turno) {

            $turnosFiltrados[] = array(
                'NMTURNO' => $turno['NMTURNO'],
                'CDTURNO' => $turno['CDTURNO'],
                'NRORG' => $turno['NRORG'],
                'CDFILIAL' => $turno['CDFILIAL'],
                'CDSERVFILI' => $turno['CDSERVFILI'],
                'COMENSAIS' => $turno[$diaDaSemanaTurno],
            );

        }

        return $turnosFiltrados;
    }

    public function retornaPontosDeEntrega(DTO\Request $request, DTO\Response $response) {

        $conditions = $request->getFilterCriteria()->getConditions();

        $cdfilial = $conditions[0]['value'];
        $cdservfili = $conditions[1]['value'];
        $dtcardapio = $conditions[2]['value'];
        $diaDaSemana = $conditions[3]['value'];

        $pontoDeEntregaCardadia = $this->retornaPontosDeEntregaCardadia($cdfilial, $cdservfili, $dtcardapio);

        if (count($pontoDeEntregaCardadia->getRows())==0) {

            $pontoDeEntregaCardadia = $this->retornaDestinos($cdfilial, $cdservfili, $diaDaSemana);
            $pontoDeEntregaCardadia = new DataSet('pontosdeentrega', $pontoDeEntregaCardadia);

        }

        $pontoDeEntregaCardadia->setDataSourceName('pontosdeentrega');

        $response->addDataSet($pontoDeEntregaCardadia);

    }

    public function retornaPontosDeEntregaCardadia($cdfilial, $cdservfili, $dtcardapio) {

        $filterCriteria = new FilterCriteria(self::CARDADIA_PONTO_ENTREGA);
        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DTCARDAPIO');

        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTCARDAPIO', $dtcardapio);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

        return $dataSet;
    }

    public function retornaDestinos($cdfilial, $cdservfili, $diaDaSemana) {

        $filterCriteria = new FilterCriteria(self::PONTO_ENTREGA_DESTINO);
        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DIADASEMANA');

        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DIADASEMANA', $diaDaSemana);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

        return $dataSet->getRows();
    }

    public function retornaComensaisEstimados(DTO\Request $request, DTO\Response $response) {

        $conditions = $request->getFilterCriteria()->getConditions();

        $cdfilial = $conditions[0]['value'];
        $cdservfili = $conditions[1]['value'];
        $diaDaSemana = $this->elaboracaoDeCardapio->retornaColunaSemana($conditions[2]['value']);

        $comensaisEstimadosServico = $this->elaboracaoDeCardapio->retornaComensaisEstimadosServico(new FilterCriteria(self::SERVICO), $cdfilial, $cdservfili);

        $comensaisEstimados = $this->elaboracaoDeCardapio->organizaDataSetComensais($comensaisEstimadosServico, $diaDaSemana);

        $response->addDataSet(new DataSet('comensaisestimados', $comensaisEstimados));

    }
    
    public function updateCardaMes($request, $response) {
        $row = $request->getRow();
        $teste = $this->elaboracaoDeCardapio->updateCardaMes($row);
    }

    public function saveCardadia(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();

        if (isset($row['NRCARDAPIO'])) {

            $this->elaboracaoDeCardapio->updateCardadia($row);

            if (isset($row['PONTODEENTREGA'])) {

                foreach ($row['PONTODEENTREGA'] as $pontoDeEntrega) {

                    $this->elaboracaoDeCardapio->salvaCardacli($pontoDeEntrega, $row['NRCARDAPIO']);
                }

            }

        } else {

            $nrcardapio = $this->elaboracaoDeCardapio->insertCardadia($row);

            if (isset($row['PONTODEENTREGA'])) {

                foreach ($row['PONTODEENTREGA'] as $pontoDeEntrega) {

                    $this->elaboracaoDeCardapio->salvaCardacli($pontoDeEntrega, $nrcardapio);
                }

            }

            $response->addDataSet(new DataSet('nrcardapio', array('nrcardapio' => $nrcardapio)));
        }
    }

    public function salvaPercentualPrato(DTO\Request $request, DTO\Response $response) {

        $rows = $request->getDataSet()->getRows();

        foreach ($rows as $row) {

            $this->elaboracaoDeCardapio->salvaPercentualPrato($row);

        }
    }

    public function salvaCardacli(DTO\Request $request, DTO\Response $response) {

        $rows = $request->getDataSet()->getRows();

        foreach ($rows as $row) {

            $this->elaboracaoDeCardapio->salvaCardacli($row);
        }
    }

    public function salvaPrato(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();

        if (isset($row['DESCENTRALIZADO']) && $row['DESCENTRALIZADO'] == 'S') {

            $row = $this->validaCompcarddel($row);

            $this->elaboracaoDeCardapio->salvaPrato($row);

        } else {

            $this->elaboracaoDeCardapio->salvaPrato($row);
        }

    }

    private function validaCompcarddel($row) {

        $compcarddel = $this->elaboracaoDeCardapio->retornaPratoExluido($row['CDFILIAL'], $row['CDSERVFILI'], $row['DATA'], $row['NRSEQPRATCAR']);

        if (count($compcarddel) > 0) {

            $row['CDPRATOMAT'] = $compcarddel[0]['CDPRATO'];
            $row['CDFILAUXPRATMAT'] = $compcarddel[0]['CDFILAUXPRAT'];
            $this->elaboracaoDeCardapio->removeCompcarddel($row['CDFILIAL'], $row['CDSERVFILI'], $row['DATA'], $row['NRSEQPRATCAR']);
        }

        return $row;
    }

    public function retornaReceita(DTO\Request $request, DTO\Response $response) {

        $this->dataSourceName = 'pla00100_receita_prato';
        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'CDPRATO', 'CDFILAUXPRAT', 'TIPOCUSTO', 'DATA', 'NRORG');
        $this->find($request, $response);
    }

    public function retornaNutrientePrato(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'pla00100_nutriente_prato';
        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'CDPRATO', 'NRORG');
        $this->find($request, $response);
    }

    public function excluirPrato(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();

        if (isset($row['DESCENTRALIZADO']) && $row['DESCENTRALIZADO'] == 'S') {

            if ($row['IDPRATOESPE'] == 'S' || $row['IDPRATOESPE'] == 'R' && isset($row['CDFILAUXPRATMAT']) && isset($row['CDPRATOMAT'])) {

                $this->elaboracaoDeCardapio->insertCompcarddel($row);
            }

            $this->elaboracaoDeCardapio->excluirPrato($row);

        } else {

            $this->elaboracaoDeCardapio->excluirPrato($row);
        }

    }

    public function copiarPratos(DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        foreach ($rows as $row) {
            if (isset($row[0]['NRCARDAPIO'])) {
                $row[1]['NRCARDAPIO'] = $row[0]['NRCARDAPIO'];
                if (isset($row[1]['DESCENTRALIZADO']) && $row[1]['DESCENTRALIZADO'] == 'S') {
                    $row[1] = $this->validaCompcarddel($row[1]);
                    $this->elaboracaoDeCardapio->salvaPrato($row[1]);
                } else {
                    $this->elaboracaoDeCardapio->salvaPrato($row[1]);
                }
            } else {
                $cdfilial = $row[1]['CDFILIAL'];
                $cdservfili = $row[1]['CDSERVFILI'];
                $dtcardapio = $row[1]['DATA'];
                if (isset($row[1]['FERIADOFILIAL']) || isset($row[1]['FERIADOCARDAPIO'])) {
                    $row[1]['DIADASEMANA'] = "7";
                }
                $diaDaSemana = $this->elaboracaoDeCardapio->retornaColunaSemana($row[1]['DIADASEMANA']);
                $comensaisEstimados = $this->retornaComensaisEstimadosCopiaPrato($cdfilial, $cdservfili, $diaDaSemana);

                if (intval($comensaisEstimados[0]['QTCOMESTICAR']) > 0) {

                    $turnos = $this->retornaTurnosCopiaPrato($cdfilial, $cdservfili, $dtcardapio, $diaDaSemana);
                    $pontosDeEntrega = $this->retornaPontosDeEntregaCopiaPrato($cdfilial, $cdservfili, $dtcardapio, $row[1]['DIADASEMANA']);

                    $row = $this->alteraRowCopiaPrato($comensaisEstimados, $turnos, $cdfilial, $cdservfili, $dtcardapio, $diaDaSemana, $row);

                    $nrcardapio = $this->elaboracaoDeCardapio->insertCardadia($row[0]);

                    if (count($pontosDeEntrega) != 0) {
                        foreach ($pontosDeEntrega as $ponto) {
                            $this->elaboracaoDeCardapio->salvaCardacli($ponto, $nrcardapio);
                        }
                    }
                    $row[1]['NRCARDAPIO'] = $nrcardapio;

                    if (isset($row['DESCENTRALIZADO']) && $row['DESCENTRALIZADO'] == 'S') {
                        $row[1] = $this->validaCompcarddel($row[1]);
                        $this->elaboracaoDeCardapio->salvaPrato($row[1]);
                    } else {
                        $this->elaboracaoDeCardapio->salvaPrato($row[1]);
                    }
                }
            }
        }
    }

    public function alteraRowCopiaPrato($comensaisEstimados, $turnos, $cdfilial, $cdservfili, $dtcardapio, $diaDaSemana, $row) {

        $row[0]['QTCOMESTICAR'] = $comensaisEstimados[0]['QTCOMESTICAR'];
        $row[0]['QTCLIESTICAR'] = $comensaisEstimados[0]['QTCLIESTICAR'];
        $row[0]['QTFUNESTICAR'] = $comensaisEstimados[0]['QTFUNESTICAR'];
        $row[0]['QTDOAESTICAR'] = $comensaisEstimados[0]['QTDOAESTICAR'];

        $row[0]['QTESTITURNO1'] = isset($turnos['QTESTITURNO1']) ? $turnos['QTESTITURNO1'] : null;
        $row[0]['QTESTITURNO2'] = isset($turnos['QTESTITURNO2']) ? $turnos['QTESTITURNO2'] : null;
        $row[0]['QTESTITURNO3'] = isset($turnos['QTESTITURNO3']) ? $turnos['QTESTITURNO3'] : null;
        $row[0]['QTESTITURNO4'] = isset($turnos['QTESTITURNO4']) ? $turnos['QTESTITURNO4'] : null;
        $row[0]['DTCARDAPIO'] = $dtcardapio;
        $row[0]['CDFILIAL'] = $cdfilial;
        $row[0]['CDSERVFILI'] = $cdservfili;

        return $row;
    }

    public function retornaTurnosCopiaPrato($cdfilial, $cdservfili, $dtcardapio, $diaDaSemana) {

        $turnos = array();
        $turnosCardadia = $this->retornaTurnosCardadia($cdfilial, $cdservfili, $dtcardapio);
        if (count($turnosCardadia->getRows()) == 0) {

            $turnosCardadia = $this->retornaTurnosServico($cdfilial, $cdservfili, $diaDaSemana);
            $turnosCardadia = new DataSet('turnos', $turnosCardadia);

        }

        if (count($turnosCardadia->getRows()) != 0) {

            foreach ($turnosCardadia->getRows() as $row) {

                $position = 'QTESTITURNO'.$row['CDTURNO'];
                $turnos[$position] = $row['COMENSAIS'];

            }
        }

        return $turnos;

    }

    public function retornaComensaisEstimadosCopiaPrato($cdfilial, $cdservfili, $diaDaSemana) {

        $comensaisEstimadosServico = $this->elaboracaoDeCardapio->retornaComensaisEstimadosServico(new FilterCriteria(self::SERVICO), $cdfilial, $cdservfili);

        return $this->elaboracaoDeCardapio->organizaDataSetComensais($comensaisEstimadosServico, $diaDaSemana);

    }

    public function retornaPontosDeEntregaCopiaPrato($cdfilial, $cdservfili, $dtcardapio, $diaDaSemana) {

        $pontos = array();
        $pontoDeEntregaCardadia = $this->retornaPontosDeEntregaCardadia($cdfilial, $cdservfili, $dtcardapio);

        if (count($pontoDeEntregaCardadia->getRows()) == 0) {

            $pontoDeEntregaCardadia = $this->retornaDestinos($cdfilial, $cdservfili, $diaDaSemana);
            $pontoDeEntregaCardadia = new DataSet('pontosdeentrega', $pontoDeEntregaCardadia);

        }

        if (count($pontoDeEntregaCardadia->getRows()) != 0) {

            foreach ($pontoDeEntregaCardadia->getRows() as $row) {

                $pontos[] = array(
                    'CDFILIAL' => $row['CDFILIAL'],
                    'CDCLIENTE' => $row['CDCLIENTE'],
                    'CDDESTFILI' => $row['CDDESTFILI'],
                    'QTCOMESTICLI' => $row['QTCOMESTICLI'],
                );

            }
        }

        return $pontos;
    }
    
     public function validaDistriuicaoCardapio ($cardapios){
        
        $cardapiosComDistribuicao;
        $result = [];
        
        foreach ($cardapios as $card) {
            
            $cardapiosComDistribuicao = $this->elaboracaoDeCardapio->verificaDistribuicao($card['CDFILIAL'], $card['NRCARDAPIO']);
            
            if($cardapiosComDistribuicao){
                $result[] = array(
                    "NRDISTCARD" => $cardapiosComDistribuicao[0]['NRDISTCARD'],
                    "CDFILIAL" => $cardapiosComDistribuicao[0]['CDFILIAL'],
                    "CDSERVFILI" => $cardapiosComDistribuicao[0]['CDSERVFILI'],
                    "NRCARDAPIO" => $cardapiosComDistribuicao[0]['NRCARDAPIO'],
                    "DTCARDAPIO" => $cardapiosComDistribuicao[0]['DTCARDAPIO']
                    );
            }
        }
        
        return isset($result) ? $result : "" ;
        
    }

    public function excluirCardapioPeriodo(DTO\Request $request, DTO\Response $response) {

        $rows = $request->getDataSet()->getRows();

        $pratosExcluir = $rows[0];
        $cardadiaExcluir = $rows[1];
        $cardapioDescentralizado = isset($rows[2]) && isset($rows[2][0]['DESCENTRALIZADO']) ? $rows[2][0]['DESCENTRALIZADO'] : false;
        
        $cardadiaComDitribuicao = $this->validaDistriuicaoCardapio($cardadiaExcluir);
        if(!$cardadiaComDitribuicao){

            try {
    
                $notification = "";
                $houveExclusao = [];
            
                foreach ($pratosExcluir as $row) {
   
                    $possuiRetirada = $this->elaboracaoDeCardapio->verificaRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);

                    if(empty($possuiRetirada)){
                        
                        $houveExclusao[] = $row['NMPRATO'];
                        
                        $row['NRSEQPRATCAR'] = $row['NRSEQCNFMES'];
                        $this->elaboracaoDeCardapio->excluirPrato($row);
        
                        if ($cardapioDescentralizado) {
        
                            if ($row['IDPRATOESPE'] == 'S' || $row['IDPRATOESPE'] == 'R' && isset($row['CDFILAUXPRATMAT']) && isset($row['CDPRATOMAT'])) {
        
                                $this->elaboracaoDeCardapio->insertCompcarddel($row);
                            }
        
                        }
                    }
    
                }
    
                foreach ($cardadiaExcluir as $row) {
    
                    $message = $this->elaboracaoDeCardapio->excluirCardadiaSemCampcard($row);
    
                    if (empty($notification) && !empty($message)) {
    
                        $notification = $message;
    
                    }
    
                }
    
                if (!empty($notification)) {
    
                    $response->addNotification(new Notification($notification, Notification::TYPE_ALERT));
                }
                
                if(!empty($houveExclusao)){
                    $response->addDataSet(new DataSet('houveexclusao', $houveExclusao));
                }
    
            } catch (\Exception $e) {
    
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        
        }else{
            $response->addDataSet(new DataSet('existeDistribuicao', $cardadiaComDitribuicao));
        }

    }

    public function retornaTodosComensaisEstimados($cdfilial, $cdservfili) {

        $comensaisEstimados = array();

        $comensaisEstimadosServico = $this->elaboracaoDeCardapio->retornaComensaisEstimadosServico(new FilterCriteria(self::SERVICO), $cdfilial, $cdservfili);


        for ($i = 0; $i <= 7; $i++) {

            $diaDaSemana = $this->elaboracaoDeCardapio->retornaColunaSemana((string) $i);
            $comensaisEstimadosDia = $this->elaboracaoDeCardapio->organizaDataSetComensais($comensaisEstimadosServico, $diaDaSemana);
            $comensaisEstimados[$diaDaSemana] = $comensaisEstimadosDia[0];

        }
        return $comensaisEstimados;

    }

    public function criaCardadiaComensaisEstimados(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $turnos = $this->retornaTurnosCopiaPrato($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $row['DIADASEMANA']);
        $pontosDeEntrega = $this->retornaPontosDeEntregaCopiaPrato($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $row['DIADASEMANA']);
        if (count($turnos) != 0) {
            foreach ($turnos as $key => $value) {
                $row[$key] = $value;
            }
        }
        $nrcardapio = $this->elaboracaoDeCardapio->insertCardadia($row);
        if (count($pontosDeEntrega) != 0) {
            foreach ($pontosDeEntrega as $ponto) {
                $this->elaboracaoDeCardapio->salvaCardacli($ponto, $nrcardapio);
            }
        }
    }

    public function retornaProdutosInvalidosReceita(DTO\Request $request, DTO\Response $response) {

        $row = $request->getRow();

        $produtos = $this->retornaProdutosReceitaInv($row);
//        $produtosInvalidos = $this->retornaProdutosInvalidos($produtos, $row['CDFILIAL']);

        $response->addDataSet(new DataSet('produtos', $produtos));

    }    

    private function retornaProdutosReceitaInv($row) {

        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'CDPRATO', 'CDFILAUXPRAT');

        $filterCriteria = new FilterCriteria('produtos_invalidos_receita');
        $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
        $filterCriteria->addCondition('CDSERVFILI', $row['CDSERVFILI']);
        $filterCriteria->addCondition('CDPRATO', $row['CDPRATO']);
        $filterCriteria->addCondition('CDFILAUXPRAT', $row['CDFILAUXPRAT']);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataset = $this->dataSourceManager->findBy($newFilterCriteria);

        return  $dataset->getRows();
    }
    
    private function retornaProdutosReceita($row) {

        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'CDPRATO', 'CDFILAUXPRAT');

        $filterCriteria = new FilterCriteria('produtos_receita');
        $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
        $filterCriteria->addCondition('CDSERVFILI', $row['CDSERVFILI']);
        $filterCriteria->addCondition('CDPRATO', $row['CDPRATO']);
        $filterCriteria->addCondition('CDFILAUXPRAT', $row['CDFILAUXPRAT']);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataset = $this->dataSourceManager->findBy($newFilterCriteria);

        $rows = $dataset->getRows();
        $produtos = array();

        foreach ($rows as $row) {
            $produtos[] = $row['CDPRODUTO'];
        }

        return implode('_', $produtos);
    }

    private function retornaProdutosInvalidos($produtos, $cdfilial) {

        $this->parameterBagColumns = array('CDFILIAL', 'NRORG', 'CDPRODUTO');

        $filterCriteria = new FilterCriteria('produtos_invalidos_receita');
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDPRODUTO', $produtos);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

        return $dataSet->getRows();
    }
    
    public function inverterPrato(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrcardapioDatas =  $this->elaboracaoDeCardapio->verificaCardapioDatas($row[0]);
        if(sizeof($nrcardapioDatas[0]) && sizeof($nrcardapioDatas[1])) {
            $row[0]['NRCARDORIG'] = $nrcardapioDatas[0][0]['NRCARDAPIO'];
            $row[0]['NRCARDDEST'] = $nrcardapioDatas[1][0]['NRCARDAPIO'];
            if($row[0]['manterComensais'] === "N") {
                $this->elaboracaoDeCardapio->inverterPratos($row[0]);
            } else {
                $this->elaboracaoDeCardapio->inverterPratosMantemComensais($row[0]);
            }
        } else {
            $response->setError(new DTO\Response\Error("Não é possível inverter pratos em um dia sem cardápio.", 1));
        }
    }
    
    public function validaDatasInverterPrato(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $response->addDataSet(new DataSet('result', $this->elaboracaoDeCardapio->validaDatasInverterPrato($row[0])));
    }
    
    public function servicoDestino(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName("pla00133_servico_destino");
        $this->parameterBagColumns = array("DTINICARDAPIO", "CDSERVFILI", "DTFINCARDAPIO", "DTMESCARDAPIO", "CDFILIAL", "CDGRUPSERV");
        $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        foreach ($filterCriteria->getConditions() as $condition) {
            if($condition['columnName'] === "CDFILIAL") {
                $cdfilial = $condition['value'];
            } else if($condition['columnName'] === "CDSERVFILI") {
                $cdservfili = $condition['value'];
            } else if($condition['columnName'] === "DTMESCARDAPIO") {
                $dtCardapio = $condition['value'];
            }
        }
        $tipoOper = $this->elaboracaoDeCardapio->retornaTipoOperador();
        $tipoCard = $this->elaboracaoDeCardapio->retornaTipoDeCardapio($cdfilial, $cdservfili);
        $paramplaneja = $this->elaboracaoDeCardapio->retornaParamplaneja($cdfilial);
        $dtprocessa = $this->elaboracaoDeCardapio->retornaDataProcPorFilial($cdfilial); 
        if(!$this->paramgeral)
           $this->paramgeral = $this->elaboracaoDeCardapio->retornaParamgeral();
        $IDUTCARDPADR = '';
        $IDPEALTCARDSERV = ''; 
        if($this->paramgeral){
           $IDUTCARDPADR = ( isset($this->paramgeral['IDUTCARDPADR']) ) ? $this->paramgeral['IDUTCARDPADR'] : '';
           $IDPEALTCARDSERV = ( isset($this->paramgeral['IDPEALTCARDSERV']) ) ? $this->paramgeral['IDPEALTCARDSERV'] : '';
        }
        $mesDtprocessa = substr($dtprocessa, 6, 4).substr($dtprocessa, 3, 2).'01'; 
        $mesCard = substr($dtCardapio, 6, 4).substr($dtCardapio, 3, 2).'01'; 
        $rows = $dataSet->getRows();
        $now = new DateTime();
        
        for ($i = 0; $i < sizeof($rows); $i++) {
            $rows[$i]['IDSITSERV'] = 'O';
            $rows[$i]['DSMENSERRROS'] = "";
            if($IDUTCARDPADR === 'S') {
                if(($rows[$i]['CARDFILICENT'] == null && $rows[$i]['IDFILCENTPME'] !== 'S') ||
                ($rows[$i]['CARDFILICENT'] !== null && $rows[$i]['IDCARDLIBCENT'] !== 'S' && $tipoOper === "FILIAL")) {
                    $rows[$i]['IDSITSERV'] = 'E';
                    $rows[$i]['DSMENSERRROS'] .= "Cardápio não liberado; ";
                }
            } else if($IDUTCARDPADR === 'D' && $tipoCard === 'S') {
                if((int)$mesCard > (int)date('Y').date('m').'01' && $rows[$i]['CTRLAPROVCARD'] != null && $rows[$i]['DTLIBECARDMATR'] == null) {
                    $rows[$i]['IDSITSERV'] = 'E';
                    $rows[$i]['DSMENSERRROS'] .= "Cardápio não liberado pela matriz; ";
                }
            }
            if($rows[$i]['CONFIGURACAO'] == null && $rows[$i]['CONFCARD'] == null) {
                $rows[$i]['IDSITSERV'] = 'E';
                $rows[$i]['DSMENSERRROS'] .= "Não existe configuração de cardápio cadastrada para o serviço informado; ";
            }
            if($rows[$i]['CARDADIA'] == null && (int)$mesCard < (int)$mesDtprocessa) {
                $rows[$i]['IDSITSERV'] = 'E';
                $rows[$i]['DSMENSERRROS'] .= "Não há cardápio para o mês de referência informado; ";
            }
            if($paramplaneja['IDCUSPADCAR'] === 'S' && ($rows[$i]['CUSTOPADRAO'] <= 0 || $rows[$i]['CUSTOPADRAO'] == null)) {
                $rows[$i]['IDSITSERV'] = 'E';
                $rows[$i]['DSMENSERRROS'] .= "Serviço não possui custo padrão cadastrado; ";
            }
            if($rows[$i]['IDCONSSERVCA'] === 'S' && $rows[$i]['CONSUMOPORSERVICO'] != null && (int)$mesCard >= (int)$mesDtprocessa) {
                $rows[$i]['IDSITSERV'] = 'E';
                $rows[$i]['DSMENSERRROS'] .= "Serviço não possui consumo fixo/percapita no período informado; ";
            }
            if($rows[$i]['IDSITSERV'] === 'O') {
                if($IDUTCARDPADR === 'S' && $rows[$i]['IDCARDLIBAGRE'] === 'S') {
                    $rows[$i]['IDSITSERV'] = 'B';
                    $rows[$i]['DSMENSERRROS'] .= "Cardápio habilitado apenas para consulta; ";
                } else if($IDUTCARDPADR === 'D' && $tipoCard === 'S') {
                    if($tipoOper === "FILIAL") {
                        if($rows[$i]['DTAPROCARDFILI'] == null) {
                            $rows[$i]['IDSITSERV'] = 'B';
                            $rows[$i]['DSMENSERRROS'] .= "Cardápio habilitado apenas para consulta; ";
                        } else if($rows[$i]['DTAPROCARDMATR'] == null) {
                            $rows[$i]['IDSITSERV'] = 'B';
                            $rows[$i]['DSMENSERRROS'] .= "Cardápio habilitado apenas para consulta; ";
                        } else if($now > DateTime::createFromFormat ('d/m/Y', $rows[$i]['DTLIMAPROVFILI'])) {
                            $rows[$i]['IDSITSERV'] = 'B';
                            $rows[$i]['DSMENSERRROS'] .= "Cardápio habilitado apenas para consulta; ";
                        }
                    } else if($tipoOper === "MATRIZ") {
                        $dtlimaprovfili = substr($rows[$i]['DTLIMAPROVFILI'], 3, 3);
                        $dtlimaprovfili .= substr($rows[$i]['DTLIMAPROVFILI'], 0, 3);
                        $dtlimaprovfili .= substr($rows[$i]['DTLIMAPROVFILI'], 6, 4);
                        if($rows[$i]['DTAPROCARDMATR'] == null && $rows[$i]['DTAPROCARDFILI'] == null && $rows[$i]['DTLIBECARDMATR'] != null &&
                        $now <= DateTime::createFromFormat ('m/d/Y', $dtlimaprovfili)) {
                            $rows[$i]['IDSITSERV'] = 'B';
                            $rows[$i]['DSMENSERRROS'] .= "Cardápio habilitado apenas para consulta; ";
                        }
                    }
                } else if($tipoOper === "FILIAL" && $IDPEALTCARDSERV === 'N') {
                    $rows[$i]['IDSITSERV'] = 'B';
                    $rows[$i]['DSMENSERRROS'] .= "Cardápio habilitado apenas para consulta; ";
                }
                if($rows[$i]['IDSITSERV'] === 'O' && $rows[$i]['CARDADIA'] == null && $rows[$i]['CICLO'] != null && (int)$mesCard > (int)date('m')) {
                    $rows[$i]['IDSITSERV'] = 'A';
                    $rows[$i]['DSMENSERRROS'] .= "Existe ciclo definido para o serviço; ";
                }
            }
        }
        $ds = new DataSet("pla00133_servico_destino", $rows);
        $response->addDataSet($ds);
    }
    
    public function exportarCardapio(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getParameters()['row'];
            $row = $this->elaboracaoDeCardapio->getExportarCardapioParams($row);
            $nrlogintcard = $this->elaboracaoDeCardapio->exportarCardapio($row, $response);
            if(isset($nrlogintcard)) {
                $logIntCard = $this->elaboracaoDeCardapio->getLogIntCard($row['CDFILIAL'], $nrlogintcard['P_NRLOGINTCARD']);
                $response->addDataSet(new DataSet("logIntCard", $logIntCard));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function atualizarDtRefCustos(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getParameters()['row'];
            $this->entityManager->beginTransaction();
            $row['CDSERVFILI'] = $row['ALLCDSERVFILI'] === 'S' ? 'T' : join('_', $row['CDSERVFILI']);
            $this->elaboracaoDeCardapio->insertDtRefeCusto($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $row['DTREFCUS']);
            $this->elaboracaoDeCardapio->updateDtRefeCusto($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $row['DTREFCUS']);
            $this->geraHistCardaMesPorServico($row);
            if($this->elaboracaoDeCardapio->existsProcedureGRAVAPRECUSTO()) {
                if($row['ALLCDSERVFILI'] === 'S') {
                    $servicos = $this->elaboracaoDeCardapio->getServicosByFilial($row['CDFILIAL']);
                    $row['CDSERVFILI'] = "";
                    foreach ($servicos as $servico) {
                        $row['CDSERVFILI'] .= $servico['CDSERVFILI'].'_';
                    }
                    if(sizeof($row['CDSERVFILI'])) {
                        $row['CDSERVFILI'] = substr($row['CDSERVFILI'], 0, -1);
                    }
                }
                $this->elaboracaoDeCardapio->executeGRAVAPRECUSTO($row);
            }
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    private function geraHistCardaMesPorServico($row) {
        if($row["CDSERVFILI"] === "T") {
            $servicos = $this->elaboracaoDeCardapio->getServicosByFilial($row['CDFILIAL']);
            foreach($servicos as $servico) {
                $rowProcedure = array(
                    "CDFILIAL"     => $row["CDFILIAL"],
                    "CDSERVFILI"   => $servico["CDSERVFILI"],
                    "DTCARDAPIO"   => $row["DTCARDAPIO"],
                    "CDCUSTOPLA"   => $row["CDCUSTOPLA"],
                    "DTREFERENCIA" => $row["DTREFERENCIA"]
                );
                $this->elaboracaoDeCardapio->geraHistCardaMes($rowProcedure);
            }
        } else {
            $servicos = explode("_", $row["CDSERVFILI"]);
            foreach($servicos as $servico) {
                $rowProcedure = array(
                    "CDFILIAL"     => $row["CDFILIAL"],
                    "CDSERVFILI"   => $servico,
                    "DTCARDAPIO"   => $row["DTCARDAPIO"],
                    "CDCUSTOPLA"   => $row["CDCUSTOPLA"],
                    "DTREFERENCIA" => $row["DTREFERENCIA"]
                );
                $this->elaboracaoDeCardapio->geraHistCardaMes($rowProcedure);
            }
        }
    }
    
    public function atualizarDtRefRecCustos(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getParameters()['row'];
            $row['CDSERVFILI'] = $row['ALLCDSERVFILI'] === 'S' ? 'T' : join('_', $row['CDSERVFILI']);
            $this->elaboracaoDeCardapio->insertDtRefeRecCusto($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $row['DTREFCUS']);
            $this->elaboracaoDeCardapio->updateDtRefeRecCusto($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $row['DTREFCUS']);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function getVRPECONSCARD($row) {
        if($row['TIPOCOMENS'] === 'C') {
            $row['VRPECONSCARD'] = ($row['COMENSDIA'] * 100) / $row['QTCOMESTICAR'];
        } else {
            $row['VRPECONSCARD'] = $row['COMENSDIA'];
        }
        return $row['VRPECONSCARD'];
    }
    
    private function getFeriados($cdfilial, $dtini) {
        $feriadosFilial = $this->elaboracaoDeCardapio->retornaDiasFeriadoFilial(new FilterCriteria(self::FERIFILI), $cdfilial, $dtini);
        $feriados = array();
        foreach ($feriadosFilial as $feriado) {
            $feriados[] = substr($feriado['DTFERIFILI'], 0, 10);
        }
        return $feriados;
    }
    
    public function copiarPratosComensais(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getParameters()['row'];
        
        $dtBloqueioSTR   = $row["DTBLOQUEIO"];
        $dtReferenciaSTR = $row["DTREFERENCIA"];
        $dtFimSTR        = $row['DTFIM'];

        $dtBloqueio   = $this->elaboracaoDeCardapio->dateToDateTime($dtBloqueioSTR);
        $dtReferencia = $this->elaboracaoDeCardapio->dateToDateTime($dtReferenciaSTR);
        $dtFim        = $this->elaboracaoDeCardapio->dateToDateTime($dtFimSTR);

        $dtInicio = $this->getDtInicio($dtReferencia, $dtBloqueio);
        $this->verificaPratosCardapio($row, $dtInicio, $dtFim, $dtBloqueio);
    }
    
    private function getDtInicio($dtReferencia, $dtBloqueio) {
        $inicioMesBloqueio = clone $dtBloqueio;
        $inicioMesBloqueio->modify('first day of this month');
        if ($dtReferencia == $inicioMesBloqueio) {
            return clone $dtBloqueio; // Copia à partir da data de bloqueio
        } else {
            return clone $dtReferencia; // Copia à partir da data de inicio do mês do cardápio
        }
    }
    private function verificaPratosCardapio($row, $dtInicio, $dtFim, $dtBloqueio) {
        $feriados = $this->getFeriados($row['CDFILIAL'], $dtInicio->format("d/m/Y"));
        
        $diasComensZero = array();
        $comensaisPorDia = $this->retornaTodosComensaisEstimados($row['CDFILIAL'], $row['CDSERVFILI']);
        $dtInicioItr = clone $dtInicio; // Iterador de dias
        for ($dtInicioItr; $dtInicioItr <= $dtFim; $dtInicioItr->add(new \DateInterval("P1D"))) {
            if (
                (!in_array($dtInicioItr->format("d/m/Y"), $feriados)) OR // Dia não é feriado
                (in_array($dtInicioItr->format("d/m/Y"), $feriados) && $row["BLOQFERIADO"] !== "S") // Dia é feriado mas não bloqueia feriados
            ) {

                if (in_array($dtInicioItr->format("d/m/Y"), $feriados)) {
                    $weekDay = self::INDEX_FERIADO;
                } else {
                    $weekDay = date('w', strtotime($dtInicioItr->format("Y-m-d")));
                }
                $row['COMENSDIA'] = $row['EXCECOES'][$weekDay] ? $row['EXCECOES'][$weekDay] : $row['COMENSAISPRATO'];
                $row["DTCARDAPIO"] = $dtInicioItr->format("d/m/Y");
                $row["DATA"] = $dtInicioItr->format("d/m/Y");

                $row = $this->verificaCardadia($row, $comensaisPorDia, $weekDay);
                if($row['valid']) {
                    $row['VRPECONSCARD'] = $row['COMENSDIA'] ? $this->getVRPECONSCARD($row) : 100;
                    $this->alteraPratoCardapio($row);
                } else {
                    $diasComensZero[] = $dtInicioItr->format("d/m/Y");
                }
            }
        }
    }
    
    private function verificaCardadia($row, $comensDia, $weekDay) {
        $cardadia = $this->elaboracaoDeCardapio->getCardaDia($row);
        if($cardadia && isset($cardadia[0])) {
            $row['NRCARDAPIO'] = $cardadia[0]['NRCARDAPIO'];
            $row['QTCOMESTICAR'] = $cardadia[0]['QTCOMESTICAR'];
            $row['QTESTITURNO1'] = $cardadia[0]['QTESTITURNO1'];
            $row['QTESTITURNO2'] = $cardadia[0]['QTESTITURNO2'];
            $row['QTESTITURNO3'] = $cardadia[0]['QTESTITURNO3'];
            $row['QTESTITURNO4'] = $cardadia[0]['QTESTITURNO4'];
            $row['QTCLIESTICAR'] = $cardadia[0]['QTCLIESTICAR'];
            $row['QTFUNESTICAR'] = $cardadia[0]['QTFUNESTICAR'];
            $row['QTDOAESTICAR'] = $cardadia[0]['QTDOAESTICAR'];
            $row['QTCOMEFETCAR'] = $cardadia[0]['QTCOMEFETCAR'];
            $row['QTCLIEFETCAR'] = $cardadia[0]['QTCLIEFETCAR'];
            $row['QTFUNEFETCAR'] = $cardadia[0]['QTFUNEFETCAR'];
            $row['QTDOAEFETCAR'] = $cardadia[0]['QTDOAEFETCAR'];
            $row['valid'] = true;
        } else {
            $row['QTCOMESTICAR'] = $comensDia[array_keys($comensDia)[$weekDay]]['QTCOMESTICAR'];
            $row['QTCLIESTICAR'] = $comensDia[array_keys($comensDia)[$weekDay]]['QTCLIESTICAR'];
            $row['QTFUNESTICAR'] = $comensDia[array_keys($comensDia)[$weekDay]]['QTFUNESTICAR'];
            $row['QTDOAESTICAR'] = $comensDia[array_keys($comensDia)[$weekDay]]['QTDOAESTICAR'];
            $row['QTESTITURNO1'] = 0;
            $row['QTESTITURNO2'] = 0;
            $row['QTESTITURNO3'] = 0;
            $row['QTESTITURNO4'] = 0;
            $row['QTCOMEFETCAR'] = 0;
            $row['QTCLIEFETCAR'] = 0;
            $row['QTFUNEFETCAR'] = 0;
            $row['QTDOAEFETCAR'] = 0;
            if($row['QTCOMESTICAR'] > 0) {
                $turnos = $this->retornaTurnosCopiaPrato($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $weekDay);
                $pontosDeEntrega = $this->retornaPontosDeEntregaCopiaPrato($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTCARDAPIO'], $weekDay);
                if (count($turnos) != 0) {
                    foreach ($turnos as $key => $value) {
                        $row[$key] = $value;
                    }
                }
                $row['NRCARDAPIO'] = $this->elaboracaoDeCardapio->insertCardadia($row);
                if (count($pontosDeEntrega) != 0) {
                    foreach ($pontosDeEntrega as $ponto) {
                        $this->elaboracaoDeCardapio->salvaCardacli($ponto, $row['NRCARDAPIO']);
                    }
                }
                $row['valid'] = true;
            } else {
                $row['valid'] = false;
            }
        }
        return $row;
    }
    
    private function alteraPratoCardapio($row) {
        try {
            $this->entityManager->beginTransaction();
            
            if($row['sobreporPratos'] === 'S') {
                if (isset($row['DESCENTRALIZADO']) && $row['DESCENTRALIZADO'] == 'S') {
                    $row = $this->validaCompcarddel($row);
                    $this->elaboracaoDeCardapio->salvaPrato($row);
                } else {
                    $this->elaboracaoDeCardapio->salvaPrato($row);
                }
            } else {
                $prato = $this->elaboracaoDeCardapio->getPrato($row);
                if(isset($prato) && isset($prato[0])) {
                    $this->elaboracaoDeCardapio->atualizaComensaisPrato($row);
                } else {
                    if (isset($row['DESCENTRALIZADO']) && $row['DESCENTRALIZADO'] == 'S') {
                        $row = $this->validaCompcarddel($row);
                        $this->elaboracaoDeCardapio->salvaPrato($row);
                    } else {
                        $this->elaboracaoDeCardapio->salvaPrato($row);
                    }
                }
            }
            
            $this->entityManager->commit();
        } catch(\Exception $e) {
            
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function copiarComensais(DTO\Request $request, DTO\Response $response) {
        $row = $request->getParameters()['row'];

        $dtBloqueioSTR   = $row["DTBLOQUEIO"];
        $dtReferenciaSTR = $row["DTREFERENCIA"];
        $dtFimSTR        = $row['DTFIM'];

        $dtBloqueio   = $this->elaboracaoDeCardapio->dateToDateTime($dtBloqueioSTR);
        $dtReferencia = $this->elaboracaoDeCardapio->dateToDateTime($dtReferenciaSTR);
        $dtFim        = $this->elaboracaoDeCardapio->dateToDateTime($dtFimSTR);

        $dtInicio = $this->getDtInicio($dtReferencia, $dtBloqueio);
        $this->verificaComensaisCardapio($row, $dtInicio, $dtFim, $dtBloqueio);
    }
    
    private function verificaComensaisCardapio($row, $dtInicio, $dtFim, $dtBloqueio) {
        $feriados = $this->getFeriados($row['CDFILIAL'], $dtInicio->format("d/m/Y"));

        $comensaisPorDia = $this->retornaTodosComensaisEstimados($row['CDFILIAL'], $row['CDSERVFILI']);
        $dtInicioItr = clone $dtInicio; // Iterador de dias
        for ($dtInicioItr; $dtInicioItr <= $dtFim; $dtInicioItr->add(new \DateInterval("P1D"))) {
            if (
                (!in_array($dtInicioItr->format("d/m/Y"), $feriados)) OR // Dia não é feriado
                (in_array($dtInicioItr->format("d/m/Y"), $feriados) && $row["BLOQFERIADO"] !== "S") // Dia é feriado mas não bloqueia feriados
            ) {
                if (in_array($dtInicioItr->format("d/m/Y"), $feriados)) {
                    $weekDay = self::INDEX_FERIADO;
                } else {
                    $weekDay = date('w', strtotime($dtInicioItr->format("Y-m-d")));
                }

                $row['DTCARDAPIO'] = $dtInicioItr->format("d/m/Y");
                $row['DATA'] = $dtInicioItr->format("d/m/Y");
                
                $prato = $this->elaboracaoDeCardapio->getPrato($row);
                if ($prato && isset($prato[0]) && isset($prato[0]['CDPRATO'])) {
                    $row['NRCARDAPIO'] = $prato[0]['NRCARDAPIO'];
                    $row['QTCOMESTICAR'] = $prato[0]['QTCOMESTICAR'];
                    $row['COMENSDIA'] = $row['EXCECOES'][$weekDay] ? $row['EXCECOES'][$weekDay] : $row['COMENSAISPRATO'];
                    $row['VRPECONSCARD'] = $row['COMENSDIA'] ? $this->getVRPECONSCARD($row) : 100;
                    $this->elaboracaoDeCardapio->atualizaComensaisPrato($row);
                }
            }
        }
    }
    
    private function verificaComensaisCardapioBloqFeriado($timeEnd, $row) {
        $feriados = $this->getFeriados($row['CDFILIAL'], $row['DTBLOQ']);
        $dt = explode('/', $row['DTBLOQ']);
        $dtbloqFormat = $dt[2].$dt[1].$dt[0];
        $comensaisPorDia = $this->retornaTodosComensaisEstimados($row['CDFILIAL'], $row['CDSERVFILI']);
        $i = 0;
        while(($dt[0]+$i <= (int)substr($row['DTFIM'], 0, 2)) && strtotime($dtbloqFormat + ($i.' days')) <= $timeEnd) {
            if(!in_array(date('d/m/Y', strtotime($dtbloqFormat + ($i.' days'))), $feriados)) {
                $row['DTCARDAPIO'] = date('d/m/Y', strtotime($dtbloqFormat + ($i.' days')));
                $row['DATA'] = date('d/m/Y', strtotime($dtbloqFormat + ($i.' days')));
                $prato = $this->elaboracaoDeCardapio->getPrato($row);
                if($prato && isset($prato[0]) && isset($prato[0]['CDPRATO'])) {
                    $row['NRCARDAPIO'] = $prato[0]['NRCARDAPIO'];
                    $row['QTCOMESTICAR'] = $prato[0]['QTCOMESTICAR'];
                    $weekDay = date('w', strtotime($dtbloqFormat + ($i.' days')));
                    $row['COMENSDIA'] = $row['EXCECOES'][$weekDay] ? $row['EXCECOES'][$weekDay] : $row['COMENSAISPRATO'];
                    $row['VRPECONSCARD'] = $row['COMENSDIA'] ? $this->getVRPECONSCARD($row) : 100;
                    $this->elaboracaoDeCardapio->atualizaComensaisPrato($row);
                }
            }
            $i++;
        }
    }
    
    private function checkCustoPadrao($paramplaneja, $cdfilial, $cdservfili, $dtrefecnfmes, $response) {
        if($paramplaneja['IDCUSPADCAR'] !== "S")
            return true;
        
        $custPadr = $this->elaboracaoDeCardapio->checkCustoPadrao($cdfilial, $cdservfili, $dtrefecnfmes);
        if(!$custPadr || !$custPadr[0] || !$custPadr[0]['VRCUSTPADR'] || $custPadr[0]['VRCUSTPADR'] < 0) {
            $response->setError(new DTO\Response\Error("Serviço não possui custo padrão cadastrado", 1));
            return false;
        }
        return true;
    }
    
    public function retornaProdutosInativos(DTO\Request $request, DTO\Response $response){

        $row = $request->getRow();

        $produtosInvalidos = $this->retornaProdutosInvalidos($row['CDPRODUTO'], $row['CDFILIAL']);

        $response->addDataSet(new DataSet('produtos', $produtosInvalidos));

    }
    
    public function retornaParametrosParaRevisao($request, $response){
        $row = $request->getRow();
        $this->authController->validationDependingGroup($request, $response);
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" =>  $row['CDFILIAL']
            )
        );
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramplaneja', $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramgeral', array());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);
        
        $conditions = array(
            array(
                "columnName" => "CDPRATO",
                "operator" => "=",
                "value" =>  $row['CDPRATO']
            ),
            array(
                "columnName" => "CDFILAUXPRAT",
                "operator" => "=",
                "value" =>  $row['CDFILAUXPRAT']
            )
        );
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('recepadr', $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);

    }
    
    public function retornaDataReferencia($request, $response) {
        $row = $request->getRow();
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" =>  $row['CDFILIAL']
            ),
            array(
                "columnName" => "CDSERVFILI",
                "operator" => "=",
                "value" =>  $row['CDSERVFILI']
            ),
            array(
                "columnName" => "DTMESCARDAPIO",
                "operator" => "=",
                "value" =>  $row['DTMESCARDAPIO']
            )
        );
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('cardames', $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        if(!empty($dataSet->getRows())) {
            $dataReferencia = $dataSet->getRows()[0]['DTREFECUSTMES'];
            $datetime = new DateTime();
            if($dataReferencia != null){
                $dataReferencia = $datetime->createFromFormat('d/m/Y H:i:s', $dataReferencia);
                $dataReferencia = $dataReferencia->format('d/m/Y');
            }
        } else {
            $conditions = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" =>  $row['CDFILIAL']
                ),
                array(
                    "columnName" => "DTMESCALENCARD",
                    "operator" => "=",
                    "value" =>  $row['DTMESCARDAPIO']
                )
            );
            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('calencard', $conditions);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            if(!empty($dataSet->getRows()) && $dataSet->getRows()[0]['DTBASCONCUSPLAN']) {
                $dataReferencia = $dataSet->getRows()[0]['DTBASCONCUSPLAN'];
            } else {
                $dataReferencia = $this->util->retornaDataProcessamento();
            }
        }
        $response->addDataSet(new DataSet('DATA', array($dataReferencia)));
    }
    
    
     public function retornacenario(DTO\Request $request, DTO\Response $response){
       $conditions = $request->getFilterCriteria()->getConditions();

        $cdfilial = $conditions[0]['value'];
        $cdservfili = $conditions[1]['value'];
        $dtrefecnfmes = $conditions[2]['value'];
        
         $cenario = $this->elaboracaoDeCardapio->checkCenario($cdfilial, $cdservfili, $dtrefecnfmes);
         $response->addDataSet(new DataSet('Cenario', $cenario));
    }
    
    public function getCardaMes($request, $response) {
        $row = $request->getRow();
        if(!isset($row['DTMESCARDAPIO'])) {
            $row['DTMESCARDAPIO'] = $row["DTREFERENCIA"];
        }
        $cardapioMes = $this->elaboracaoDeCardapio->getCardaMes($row['CDFILIAL'],$row['CDSERVFILI'],$row['DTMESCARDAPIO']);
        
        if(empty($cardapioMes)) {
            $this->elaboracaoDeCardapio->insertCardames($row['CDFILIAL'],$row['CDSERVFILI'],$row['DTMESCARDAPIO']);
        }
        
        $response->addDataSet(new DataSet('CardapioMes', $cardapioMes));
    }
}