<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class PLA11000NutrientesDoCardapioVetController extends \Zeedhi\Framework\Controller\Report {

    protected $novoCodigo;
    protected $procedureNutriente;
    protected $parameterBagColumns;
    protected $entityManager;
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $nrsequnutr;
    protected $cdfilial;
    protected $nrorg;
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $dataSourceManager;

    public function __construct($reportService, $novoCodigo, $entityManager, $qr2Strategy, $birtStrategy,$environment, $connection,$crudService,$dataSourceManager) {
        
        $this->reportService      = $reportService;
        $this->novoCodigo         = $novoCodigo;
        $this->entityManager      = $entityManager;
        $this->qr2Strategy        = $qr2Strategy;
        $this->birtStrategy       = $birtStrategy;
        $this->environment        = $environment;
        $this->connection         = $connection;
        $this->crudService        = $crudService;
        $this->dataSourceManager  = $dataSourceManager;
    }

    protected $reportMapping = array (
    	'PLA11000' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (

                'NRLISTA'     => 'P_NRLISTA',
                'DTINICIAL'      => 'P_DTINI',
                'DTFINAL'      => 'P_DTFIN',
                'NRORG'     =>'P_NRORG'
            ),
        ),
		'PLA11001' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (

                'NRLISTA'     => 'P_NRLISTA',
                'DTINICIAL'      => 'P_DTINI',
                'DTFINAL'      => 'P_DTFIN',
                 'NRORG'     =>'P_NRORG'
            ),
        ),
		'PLA11002' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (

                'NRLISTA'     => 'P_NRLISTA',
                'DTINICIAL'      => 'P_DTINI',
                'DTFINAL'      => 'P_DTFIN',
                  'NRORG'     =>'P_NRORG'
            ),
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
       
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $row['REPORT_NAME'];
        $row['__report_name']= $this->stNomeRel;
        $this->getFiliais($request, $response);
        //cria lista de FILIAL concatenada com NRSEQUNUTR
        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
       
        
        for ($i=0;$i<count($row['CDFILIAL']);$i++){
			$this->cdfilial = $row['CDFILIAL'][$i];
		
			if($row['NMSERVICO']=='T'){
                $row['L_SERVICO']=$this->retornaServicos($row);
			}else{
			    $row['L_SERVICO']=$this->retornaServicosSelecionados($row);
			}
            $row['L_NUTRI'] = $this->getNutrientes();
            $row['NRSEQUNUTR'] = $this->novoCodigo->getCodeByTableName('CALCNUTR'.$row['CDFILIAL'][$i], $row['NRORG']);
            $row['IDNDPCAL']='N';        

            //adiciona FILIAL concatenada com NRSEQUNUTR na lista 
            $this->insereLista($nrlista,$row['CDFILIAL'][$i].$row['NRSEQUNUTR']);
            $procedureNutriente = new StoredProcedure($this->connection, 'NUTRIENTE_DO_CARDAPIO_PLA');
            
            $procedureNutriente->addParam(new Param('P_FILIAL',     Param::PARAM_INPUT, $row['CDFILIAL'][$i],     Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_SERVICOS',   Param::PARAM_INPUT, $row['L_SERVICO'],   Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_NUTRIENTES', Param::PARAM_INPUT, $row['L_NUTRI'],     Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_NRSEQUNUTR', Param::PARAM_INPUT, $row['NRSEQUNUTR'],   Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_IDNDPCAL',   Param::PARAM_INPUT, $row['IDNDPCAL'], Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_DTINICIAL',  Param::PARAM_INPUT, $row['DTINICIAL'],      Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_DTFINAL',    Param::PARAM_INPUT, $row['DTFINAL'],      Param::PARAM_TYPE_STR));
            $procedureNutriente->execute();
        }
        $row['NRLISTA']= $nrlista ;
    }
	protected function getFiliais($request, $response){
		$row = $request->getRow();
		if((is_array($row['CDFILIAL'])!=true) && $row['CDFILIAL']=="T"){
			if(!empty($row['CDGRUPFILI'])){
				$conditions= array(
					array(
						'columnName' => 'CDGRUPFILI',
						'operator' => '=',
						'value' => $row['CDGRUPFILI']
					),
					array(
						"columnName" =>'NRORG',
						"operator" => "=",
						"value" => $this->environment->getNrOrgTrab()
					),
					array(
						"columnName" =>'IDCNTRRESC',
						"operator" => "=",
						"value" => 'N'
					)
				);
				
				
				$this->parameterBagColumns = array("CDGRUPFILI", "NRORG");
				$filterCriteria = new FilterCriteria('filial_filgrupfili', $conditions);
				$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
				$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
			} else{
				$conditions=array();
				$filterCriteria = new FilterCriteria('filial', $conditions);
            	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
			}
			$row['CDFILIAL']=array();
			foreach ($dataSet->getRows() as $filial) {
				array_push($row['CDFILIAL'], $filial['CDFILIAL']);
			}
		}
	}
	public function retornaServicos($row) {
        $sql = <<<SQL
             SELECT DISTINCT  
    s.cdfilial,
    s.cdservfili
    FROM
        filial        f
        JOIN servico       s ON s.cdfilial = f.cdfilial
        LEFT JOIN grupserv      g ON g.cdgrupserv = s.cdgrupserv
        LEFT JOIN filgrupfili   gf ON gf.cdfilial = s.cdfilial
    WHERE
        ( ( :CDFILIAL = 'T' )
          OR ( instr(:CDFILIAL, s.cdfilial) <> 0 ) )
        AND ( ( :CDGRUPSERV = 'T' )
              OR ( instr(:CDGRUPSERV, s.cdgrupserv) <> 0 ) )
    
    ORDER BY
        s.cdfilial,
        s.cdservfili
SQL;
        $params = array(
            'CDFILIAL'     => $this->cdfilial,
            'CDGRUPSERV'   => $row['GRUPSERV']
        );
        
        try {
            $lista = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result='';
            for($j=0;$j<count($lista);$j++){
                $result=$result.$lista[$j]['CDSERVFILI'].'_';
            }

            return  $result=substr($result, 0, -1);
        } catch (\Exception $e) {
            throw $e;
        }
    }
	public function getNutrientes(){
	    $sql = "SELECT CDCALORIAS, CDPROTEINA, CDCARBOHIDRA, CDALCOOL, CDFIBRAS, CDSODIO, CDLIPIDIOS, CDGORDURASATURADA FROM PARAMGERAL";
        $nutrientes = $this->entityManager->getConnection()->fetchAll($sql);
        $nutrientes =  implode('_', $nutrientes[0]);
        return $nutrientes.'_';
	}
    private function insereLista($nrlista, $valor) {
		$row = array();

			array_push($row,
				array(
					"NRLISTA" 	=> $nrlista,
					"CDLISTA" 	=> $valor,
					"__is_new"  => true
				)
			);

		$dataSet = new DataSet(null, $row);
		$request = new DTO\Request\DataSet($dataSet, '', '', '');
		$rowsToSave = $this->crudService->getRowsToSave($request, 'lista');
		$this->crudService->save('lista', $rowsToSave);
	}
	public function retornaServicosSelecionados($row){
	    $servicos='';
	    for($i=0;$i<count($row['NMSERVICO']);$i++){
	        if($row['NMSERVICO'][$i]['CDFILIAL']== $this->cdfilial){
	            $servicos=$servicos.$row['NMSERVICO'][$i]['CDSERVFILI'].'_';
	        }
	    }
	    return $servicos=substr($servicos, 0, -1);
	}
    
}